// zig fmt: off
const std = @import("std");
const book = @import("book.zig");

/// 与えられたバッファがePubアーカイブであるかどうかを返します。
pub fn isEpub(buf: []const u8) bool {
  return book.isEpub(buf);
}

/// 与えられたバッファがZIPアーカイブであるかどうかを返します。
pub fn isZip(buf: []const u8) bool {
  return buf.len > 3 and
    buf[0] == 0x50 and
    buf[1] == 0x4B and
    (((buf[2] == 0x3 and buf[3] == 0x04) or
      (buf[2] == 0x5 and buf[3] == 0x06) or
      (buf[2] == 0x7 and buf[3] == 0x08)) or
      (
        // winzip
        buf.len > 7 and
        (buf[2] == 0x30 and
         buf[3] == 0x30 and
         buf[4] == 0x50 and
         buf[5] == 0x4B and
         buf[6] == 0x03 and
         buf[7] == 0x04)
    ));
}

/// 与えられたバッファがTarアーカイブであるかどうかを返します。
pub fn isTar(buf: []const u8) bool {
  return buf.len > 261 and
    buf[257] == 0x75 and
    buf[258] == 0x73 and
    buf[259] == 0x74 and
    buf[260] == 0x61 and
    buf[261] == 0x72;
}

/// 与えられたバッファがPar2アーカイブであるかどうかを返します。
pub fn isPar2(buf: []const u8) bool {
  return buf.len > 8 and
    buf[0] == 0x50 and
    buf[1] == 0x41 and
    buf[2] == 0x52 and
    buf[3] == 0x32 and
    buf[4] == 0x00 and
    buf[5] == 0x50 and
    buf[6] == 0x4B and
    buf[7] == 0x54;
}

/// 与えられたバッファがRARアーカイブであるかどうかを返します。
pub fn isRar(buf: []const u8) bool {
  return buf.len > 6 and
    buf[0] == 0x52 and
    buf[1] == 0x61 and
    buf[2] == 0x72 and
    buf[3] == 0x21 and
    buf[4] == 0x1A and
    buf[5] == 0x07 and
    (buf[6] == 0x00 or buf[6] == 0x01);
}

/// 与えられたバッファがGzipアーカイブであるかどうかを返します。
pub fn isGz(buf: []const u8) bool {
  return buf.len > 2 and
    buf[0] == 0x1F and
    buf[1] == 0x8B and
    buf[2] == 0x08;
}

/// 与えられたバッファがBZIP2アーカイブであるかどうかを返します。
pub fn isBz2(buf: []const u8) bool {
  return buf.len > 2 and
    buf[0] == 0x42 and
    buf[1] == 0x5A and
    buf[2] == 0x68;
}

/// 与えられたバッファがBZIP3アーカイブであるかどうかを返します。
pub fn isBz3(buf: []const u8) bool {
  return buf.len > 4 and
    buf[0] == 'B' and
    buf[1] == 'Z' and
    buf[2] == '3' and
    buf[3] == 'v' and
    buf[4] == '1';
}

/// 与えられたバッファが7zアーカイブであるかどうかを返します。
pub fn is7z(buf: []const u8) bool {
  return buf.len > 5 and
    buf[0] == 0x37 and
    buf[1] == 0x7A and
    buf[2] == 0xBC and
    buf[3] == 0xAF and
    buf[4] == 0x27 and
    buf[5] == 0x1C;
}

/// 与えられたバッファがPDFアーカイブであるかどうかを返します。
pub fn isPdf(buf: []const u8) bool {
  return buf.len > 3 and
    buf[0] == 0x25 and
    buf[1] == 0x50 and
    buf[2] == 0x44 and
    buf[3] == 0x46;
}

/// 与えられたバッファがSWFアーカイブであるかどうかを返します。
pub fn isSwf(buf: []const u8) bool {
  return buf.len > 2 and
    (buf[0] == 0x43 or buf[0] == 0x46) and
     buf[1] == 0x57 and
     buf[2] == 0x53;
}

/// 与えられたバッファがRTFアーカイブであるかどうかを返します。
/// Returns whether a buffer is an RTF.
pub fn isRtf(buf: []const u8) bool {
  return buf.len > 4 and
    buf[0] == 0x7B and
    buf[1] == 0x5C and
    buf[2] == 0x72 and
    buf[3] == 0x74 and
    buf[4] == 0x66;
}

/// 与えられたバッファがGoogle Chrome延長機能アーカイブであるかどうかを返します。
pub fn isCrx(buf: []const u8) bool {
  return buf.len > 3 and
    buf[0] == 0x43 and
    buf[1] == 0x72 and
    buf[2] == 0x32 and
    buf[3] == 0x34;
}

/// 与えられたバッファがCABアーカイブであるかどうかを返します。
pub fn isCab(buf: []const u8) bool {
  return buf.len > 3 and
    ((buf[0] == 0x4D and buf[1] == 0x53 and buf[2] == 0x43 and buf[3] == 0x46) or
     (buf[0] == 0x49 and buf[1] == 0x53 and buf[2] == 0x63 and buf[3] == 0x28));
}

/// 与えられたバッファがEOT Eotot Streamアーカイブであるかどうかを返します。
pub fn isEot(buf: []const u8) bool {
  return buf.len > 35 and
    buf[34] == 0x4C and
    buf[35] == 0x50 and
    ((buf[8] == 0x02 and buf[9] == 0x00 and buf[10] == 0x01) or
     (buf[8] == 0x01 and buf[9] == 0x00 and buf[10] == 0x00) or
     (buf[8] == 0x02 and buf[9] == 0x00 and buf[10] == 0x02));
}

/// 与えられたバッファがPostScriptアーカイブであるかどうかを返します。
pub fn isPs(buf: []const u8) bool {
  return buf.len > 1 and
    buf[0] == 0x25 and
    buf[1] == 0x21;
}

/// 与えられたバッファがXZアーカイブであるかどうかを返します。
pub fn isXz(buf: []const u8) bool {
  return buf.len > 5 and
    buf[0] == 0xFD and
    buf[1] == 0x37 and
    buf[2] == 0x7A and
    buf[3] == 0x58 and
    buf[4] == 0x5A and
    buf[5] == 0x00;
}

/// 与えられたバッファがSQLite3データベースであるかどうかを返します。
pub fn isSqlite(buf: []const u8) bool {
  return buf.len > 3 and
    buf[0] == 0x53 and
    buf[1] == 0x51 and
    buf[2] == 0x4C and
    buf[3] == 0x69;
}

/// 与えられたバッファがDEBアーカイブであるかどうかを返します。
pub fn isDeb(buf: []const u8) bool {
  return buf.len > 20 and
    buf[0] == 0x21 and
    buf[1] == 0x3C and
    buf[2] == 0x61 and
    buf[3] == 0x72 and
    buf[4] == 0x63 and
    buf[5] == 0x68 and
    buf[6] == 0x3E and
    buf[7] == 0x0A and
    buf[8] == 0x64 and
    buf[9] == 0x65 and
    buf[10] == 0x62 and
    buf[11] == 0x69 and
    buf[12] == 0x61 and
    buf[13] == 0x6E and
    buf[14] == 0x2D and
    buf[15] == 0x62 and
    buf[16] == 0x69 and
    buf[17] == 0x6E and
    buf[18] == 0x61 and
    buf[19] == 0x72 and
    buf[20] == 0x79;
}

/// 与えられたバッファがArアーカイブであるかどうかを返します。
pub fn isAr(buf: []const u8) bool {
  return buf.len > 6 and
    buf[0] == 0x21 and
    buf[1] == 0x3C and
    buf[2] == 0x61 and
    buf[3] == 0x72 and
    buf[4] == 0x63 and
    buf[5] == 0x68 and
    buf[6] == 0x3E;
}

/// 与えられたバッファがZアーカイブであるかどうかを返します。
pub fn isZ(buf: []const u8) bool {
  return buf.len > 1 and
    buf[0] == 0x1F and
    (buf[1] == 0xA0 or buf[1] == 0x9D);
}

/// 与えられたバッファがLZipアーカイブであるかどうかを返します。
pub fn isLz(buf: []const u8) bool {
  return buf.len > 3 and
    buf[0] == 0x4C and
    buf[1] == 0x5A and
    buf[2] == 0x49 and
    buf[3] == 0x50;
}

/// 与えられたバッファがRPMアーカイブであるかどうかを返します。
pub fn isRpm(buf: []const u8) bool {
  return buf.len > 96 and
    buf[0] == 0xED and
    buf[1] == 0xAB and
    buf[2] == 0xEE and
    buf[3] == 0xDB;
}

/// 与えられたバッファがDCMアーカイブであるかどうかを返します。
pub fn isDcm(buf: []const u8) bool {
  return buf.len > 131 and
    buf[128] == 0x44 and
    buf[129] == 0x49 and
    buf[130] == 0x43 and
    buf[131] == 0x4D;
}

const ZSTD_SKIP_START: usize = 0x184D2A50;
const ZSTD_SKIP_MASK: usize = 0xFFFFFFF0;

/// 与えられたバッファがZstdアーカイブであるかどうかを返します。
pub fn isZst(buf: []const u8) bool {
  if (buf.len > 3 and
      buf[0] == 0x28 and
      buf[1] == 0xB5 and
      buf[2] == 0x2F and
      buf[3] == 0xFD) return true;

  if (buf.len < 8) return false;
  const magic = std.mem.readInt(u32, buf[0..4], .little);
  const magicUsize = @as(usize, magic);
  if (magicUsize & ZSTD_SKIP_MASK != ZSTD_SKIP_START) {
    return false;
  }

  const dataLen = std.mem.readInt(u32, buf[4..8], .little);
  const dataLenUsize = @as(usize, dataLen);
  if (buf.len < 8 + dataLenUsize) {
    return false;
  }

  const nextFrame = buf[8 + dataLenUsize..];
  return isZst(nextFrame);
}

/// 与えられたバッファがLZ4アーカイブであるかどうかを返します。
pub fn isLz4(buf: []const u8) bool {
  if (buf.len > 3 and
      buf[0] == 0x04 and
      buf[1] == 0x22 and
      buf[2] == 0x4D and
      buf[3] == 0x18) return true;

  if (buf.len < 8) return false;
  const magic = std.mem.readInt(u32, buf[0..4], .little);
  const magicUsize = @as(usize, magic);
  if (magicUsize & ZSTD_SKIP_MASK != ZSTD_SKIP_START) {
    return false;
  }

  const dataLen = std.mem.readInt(u32, buf[4..8], .little);
  const dataLenUsize = @as(usize, dataLen);
  if (buf.len < 8 + dataLenUsize) {
    return false;
  }

  const nextFrame = buf[8 + dataLenUsize..];
  return isLz4(nextFrame);
}

/// 与えられたバッファがMSI Windows インストーラーアーカイブであるかどうかを返します。
pub fn isMsi(buf: []const u8) bool {
  return buf.len > 7 and
    buf[0] == 0xD0 and
    buf[1] == 0xCF and
    buf[2] == 0x11 and
    buf[3] == 0xE0 and
    buf[4] == 0xA1 and
    buf[5] == 0xB1 and
    buf[6] == 0x1A and
    buf[7] == 0xE1;
}

/// 与えられたバッファがCPIOアーカイブであるかどうかを返します。
pub fn isCpio(buf: []const u8) bool {
  return (buf.len > 1 and
    ((buf[0] == 0xC7 and buf[1] == 0x71) or // LE
     (buf[0] == 0x71 and buf[1] == 0xC7))) or // BE
    (buf.len > 6 and
     buf[0] == 0x30 and
     buf[1] == 0x37 and
     buf[2] == 0x30 and
     buf[3] == 0x37 and
     buf[4] == 0x30 and
     buf[5] == 0x31);
}

test "is zst" {
  const zstBuf = [_]u8{ 0x28, 0xB5, 0x2F, 0xFD, 0x00 };
  try std.testing.expect(isZst(&zstBuf));

  var skipBuf = [_]u8{0} ** 16;
  skipBuf[0] = 0x50; // 魔法: 0x184D2A50 (little-endian)
  skipBuf[1] = 0x2A;
  skipBuf[2] = 0x4D;
  skipBuf[3] = 0x18;
  skipBuf[4] = 0x04; // データの長さ: 4 バイト
  skipBuf[5] = 0x00;
  skipBuf[6] = 0x00;
  skipBuf[7] = 0x00;
  skipBuf[8] = 0x00; // スキップしたデータ (4 バイト)
  skipBuf[9] = 0x00;
  skipBuf[10] = 0x00;
  skipBuf[11] = 0x00;
  skipBuf[12] = 0x28; // 次のフレーム: Zstdの魔法
  skipBuf[13] = 0xB5;
  skipBuf[14] = 0x2F;
  skipBuf[15] = 0xFD;
  try std.testing.expect(isZst(&skipBuf));

  const invalid = [_]u8{ 0x29, 0xB5, 0x2F, 0xFD };
  try std.testing.expect(!isZst(&invalid));

  const short = [_]u8{ 0x28, 0xB5, 0x2F };
  try std.testing.expect(!isZst(&short));

  const shortSkipBuf = [_]u8{ 0x50, 0x2A, 0x4D, 0x18, 0x04, 0x00, 0x00, 0x00 };
  try std.testing.expect(!isZst(&shortSkipBuf));
}

test "is lz4" {
  const lz4Buf = [_]u8{ 0x04, 0x22, 0x4D, 0x18, 0x00 };
  try std.testing.expect(isLz4(&lz4Buf));

  var skipBuf = [_]u8{0} ** 16;
  skipBuf[0] = 0x50; // 魔法: 0x184D2A50 (little-endian)
  skipBuf[1] = 0x2A;
  skipBuf[2] = 0x4D;
  skipBuf[3] = 0x18;
  skipBuf[4] = 0x04;// データの長さ: 4 バイト
  skipBuf[5] = 0x00;
  skipBuf[6] = 0x00;
  skipBuf[7] = 0x00;
  skipBuf[8] = 0x00; // スキップしたデータ (4 バイト)
  skipBuf[9] = 0x00;
  skipBuf[10] = 0x00;
  skipBuf[11] = 0x00;
  skipBuf[12] = 0x04; // 次のフレーム: LZ4の魔法
  skipBuf[13] = 0x22;
  skipBuf[14] = 0x4D;
  skipBuf[15] = 0x18;
  try std.testing.expect(isLz4(&skipBuf));

  const invalid = [_]u8{ 0x05, 0x22, 0x4D, 0x18 };
  try std.testing.expect(!isLz4(&invalid));

  const short = [_]u8{ 0x04, 0x22, 0x4D };
  try std.testing.expect(!isLz4(&short));

  const shortSkipBuf = [_]u8{ 0x50, 0x2A, 0x4D, 0x18, 0x04, 0x00, 0x00, 0x00 };
  try std.testing.expect(!isLz4(&shortSkipBuf));
}