// zig fmt: off
const std = @import("std");

/// 与えられたバッファがePubであるかどうかを返します。
pub fn isEpub(buf: []const u8) bool {
  return buf.len > 57 and
    buf[0] == 0x50 and // P
    buf[1] == 0x4B and // K
    buf[2] == 0x03 and
    buf[3] == 0x04 and
    buf[30] == 0x6D and // m
    buf[31] == 0x69 and // i
    buf[32] == 0x6D and // m
    buf[33] == 0x65 and // e
    buf[34] == 0x74 and // t
    buf[35] == 0x79 and // y
    buf[36] == 0x70 and // p
    buf[37] == 0x65 and // e
    buf[38] == 0x61 and // a
    buf[39] == 0x70 and // p
    buf[40] == 0x70 and // p
    buf[41] == 0x6C and // l
    buf[42] == 0x69 and // i
    buf[43] == 0x63 and // c
    buf[44] == 0x61 and // a
    buf[45] == 0x74 and // t
    buf[46] == 0x69 and // i
    buf[47] == 0x6F and // o
    buf[48] == 0x6E and // n
    buf[49] == 0x2F and // /
    buf[50] == 0x65 and // e
    buf[51] == 0x70 and // p
    buf[52] == 0x75 and // u
    buf[53] == 0x62 and // b
    buf[54] == 0x2B and // +
    buf[55] == 0x7A and // z
    buf[56] == 0x69 and // i
    buf[57] == 0x70;    // p
}

/// 与えられたバッファがmobiであるかどうかを返します。
pub fn isMobi(buf: []const u8) bool {
  return buf.len > 67 and
    buf[60] == 0x42 and // B
    buf[61] == 0x4F and // O
    buf[62] == 0x4F and // O
    buf[63] == 0x4B and // K
    buf[64] == 0x4D and // M
    buf[65] == 0x4F and // O
    buf[66] == 0x42 and // B
    buf[67] == 0x49;    // I
}

test "is epub" {
  var epubBuf = [_]u8{0} ** 58;
  epubBuf[0] = 0x50;
  epubBuf[1] = 0x4B;
  epubBuf[2] = 0x03;
  epubBuf[3] = 0x04;
  const mimetype = "mimetypeapplication/epub+zip";
  for (mimetype, 0..) |c, i| {
    epubBuf[30 + i] = c;
  }
  try std.testing.expect(isEpub(&epubBuf));

  var invalid = epubBuf;
  invalid[30] = 0x6E;
  try std.testing.expect(!isEpub(&invalid));

  const short = [_]u8{0} ** 57;
  try std.testing.expect(!isEpub(&short));
}

test "is mobi" {
  var mobiBuf = [_]u8{0} ** 68;
  mobiBuf[60] = 0x42; // B
  mobiBuf[61] = 0x4F; // O
  mobiBuf[62] = 0x4F; // O
  mobiBuf[63] = 0x4B; // K
  mobiBuf[64] = 0x4D; // M
  mobiBuf[65] = 0x4F; // O
  mobiBuf[66] = 0x42; // B
  mobiBuf[67] = 0x49; // I
  try std.testing.expect(isMobi(&mobiBuf));

  // Negative case: invalid signature
  var invalid = mobiBuf;
  invalid[60] = 0x43; // Change 'B' to 'C'
  try std.testing.expect(!isMobi(&invalid));

  // Edge case: too short
  const short = [_]u8{0} ** 67;
  try std.testing.expect(!isMobi(&short));
}