// zig fmt: off
const std = @import("std");

/// 与えられたバッファがWOFFフォントデータであるかどうかを返します。
pub fn isWoff(buf: []const u8) bool {
  return buf.len > 7 and
    buf[0] == 0x77 and
    buf[1] == 0x4F and
    buf[2] == 0x46 and
    buf[3] == 0x46 and
    buf[4] == 0x00 and
    buf[5] == 0x01 and
    buf[6] == 0x00 and
    buf[7] == 0x00;
}

/// 与えられたバッファがWOFF2フォントデータであるかどうかを返します。
pub fn isWoff2(buf: []const u8) bool {
  return buf.len > 7 and
    buf[0] == 0x77 and
    buf[1] == 0x4F and
    buf[2] == 0x46 and
    buf[3] == 0x32 and
    buf[4] == 0x00 and
    buf[5] == 0x01 and
    buf[6] == 0x00 and
    buf[7] == 0x00;
}

/// 与えられたバッファがTTFフォントデータであるかどうかを返します。
pub fn isTtf(buf: []const u8) bool {
  return buf.len > 4 and
    buf[0] == 0x00 and
    buf[1] == 0x01 and
    buf[2] == 0x00 and
    buf[3] == 0x00 and
    buf[4] == 0x00;
}

/// 与えられたバッファがOTFフォントデータであるかどうかを返します。
pub fn isOtf(buf: []const u8) bool {
  return buf.len > 4 and
    buf[0] == 0x4F and
    buf[1] == 0x54 and
    buf[2] == 0x54 and
    buf[3] == 0x4F and
    buf[4] == 0x00;
}

test "is woff" {
  const woffBuf = [_]u8{ 0x77, 0x4F, 0x46, 0x46, 0x00, 0x01, 0x00, 0x00, 0x00 };
  try std.testing.expect(isWoff(&woffBuf));

  const invalid = [_]u8{ 0x77, 0x4F, 0x46, 0xFF, 0x00, 0x01, 0x00, 0x00 };
  try std.testing.expect(!isWoff(&invalid));

  const short = [_]u8{ 0x77, 0x4F, 0x46, 0x46, 0x00, 0x01, 0x00 };
  try std.testing.expect(!isWoff(&short));

  const empty = [_]u8{};
  try std.testing.expect(!isWoff(&empty));
}

test "is woff2" {
  const woff2Buf = [_]u8{ 0x77, 0x4F, 0x46, 0x32, 0x00, 0x01, 0x00, 0x00, 0x00 };
  try std.testing.expect(isWoff2(&woff2Buf));

  const invalid = [_]u8{ 0x77, 0x4F, 0x46, 0xFF, 0x00, 0x01, 0x00, 0x00 };
  try std.testing.expect(!isWoff2(&invalid));

  const short = [_]u8{ 0x77, 0x4F, 0x46, 0x32, 0x00, 0x01, 0x00 };
  try std.testing.expect(!isWoff2(&short));

  const empty = [_]u8{};
  try std.testing.expect(!isWoff2(&empty));
}

test "is ttf" {
  const ttfBuf = [_]u8{ 0x00, 0x01, 0x00, 0x00, 0x00, 0x00 };
  try std.testing.expect(isTtf(&ttfBuf));

  const invalid = [_]u8{ 0x00, 0x01, 0x00, 0xFF, 0x00 };
  try std.testing.expect(!isTtf(&invalid));

  const short = [_]u8{ 0x00, 0x01, 0x00, 0x00 };
  try std.testing.expect(!isTtf(&short));

  const empty = [_]u8{};
  try std.testing.expect(!isTtf(&empty));
}

test "is otf" {
  const otfBuf = [_]u8{ 0x4F, 0x54, 0x54, 0x4F, 0x00, 0x00 };
  try std.testing.expect(isOtf(&otfBuf));

  const invalid = [_]u8{ 0x4F, 0x54, 0x54, 0xFF, 0x00 };
  try std.testing.expect(!isOtf(&invalid));

  const short = [_]u8{ 0x4F, 0x54, 0x54, 0x4F };
  try std.testing.expect(!isOtf(&short));

  const empty = [_]u8{};
  try std.testing.expect(!isOtf(&empty));
}