// zig fmt: off
const std = @import("std");

/// 与えられたバッファがM4V動画データであるかどうかを返します。
pub fn isM4v(buf: []const u8) bool {
  return buf.len > 10 and
    buf[4] == 0x66 and
    buf[5] == 0x74 and
    buf[6] == 0x79 and
    buf[7] == 0x70 and
    buf[8] == 0x4D and
    buf[9] == 0x34 and
    buf[10] == 0x56;
}

/// 与えられたバッファがMKV動画データであるかどうかを返します。
pub fn isMkv(buf: []const u8) bool {
  return (buf.len > 15 and
    buf[0] == 0x1A and
    buf[1] == 0x45 and
    buf[2] == 0xDF and
    buf[3] == 0xA3 and
    buf[4] == 0x93 and
    buf[5] == 0x42 and
    buf[6] == 0x82 and
    buf[7] == 0x88 and
    buf[8] == 0x6D and
    buf[9] == 0x61 and
    buf[10] == 0x74 and
    buf[11] == 0x72 and
    buf[12] == 0x6F and
    buf[13] == 0x73 and
    buf[14] == 0x6B and
    buf[15] == 0x61) or
    (buf.len > 38 and
      buf[31] == 0x6D and
      buf[32] == 0x61 and
      buf[33] == 0x74 and
      buf[34] == 0x72 and
      buf[35] == 0x6f and
      buf[36] == 0x73 and
      buf[37] == 0x6B and
      buf[38] == 0x61);
}

/// 与えられたバッファがWEBM動画データであるかどうかを返します。
pub fn isWebm(buf: []const u8) bool {
  return buf.len > 3 and
    buf[0] == 0x1A and
    buf[1] == 0x45 and
    buf[2] == 0xDF and
    buf[3] == 0xA3;
}

/// 与えられたバッファがQuicktime MOV動画データであるかどうかを返します。
pub fn isMov(buf: []const u8) bool {
  return buf.len > 15 and
    (((buf[4] == 'f' and buf[5] == 't' and buf[6] == 'y' and buf[7] == 'p') and
      (buf[8] == 'q' and buf[9] == 't' and buf[10] == ' ' and buf[11] == ' ')) or
      (buf[4] == 0x6D and buf[5] == 0x6F and buf[6] == 0x6F and buf[7] == 0x76) or
      (buf[4] == 0x6D and buf[5] == 0x64 and buf[6] == 0x61 and buf[7] == 0x74) or
      (buf[12] == 0x6D and buf[13] == 0x64 and buf[14] == 0x61 and buf[15] == 0x74));
}

/// 与えられたバッファがAVI動画データであるかどうかを返します。
pub fn isAvi(buf: []const u8) bool {
  return buf.len > 10 and
    buf[0] == 0x52 and
    buf[1] == 0x49 and
    buf[2] == 0x46 and
    buf[3] == 0x46 and
    buf[8] == 0x41 and
    buf[9] == 0x56 and
    buf[10] == 0x49;
}

/// 与えられたバッファがWMV動画データであるかどうかを返します。
pub fn isWmv(buf: []const u8) bool {
  return buf.len > 9 and
    buf[0] == 0x30 and
    buf[1] == 0x26 and
    buf[2] == 0xB2 and
    buf[3] == 0x75 and
    buf[4] == 0x8E and
    buf[5] == 0x66 and
    buf[6] == 0xCF and
    buf[7] == 0x11 and
    buf[8] == 0xA6 and
    buf[9] == 0xD9;
}

/// 与えられたバッファがMPEG動画データであるかどうかを返します。
pub fn isMpeg(buf: []const u8) bool {
  return buf.len > 3 and
    buf[0] == 0x00 and
    buf[1] == 0x00 and
    buf[2] == 0x01 and
    buf[3] >= 0xB0 and
    buf[3] <= 0xBF;
}

/// 与えられたバッファがFLV動画データであるかどうかを返します。
pub fn isFlv(buf: []const u8) bool {
  return buf.len > 3 and
    buf[0] == 0x46 and
    buf[1] == 0x4C and
    buf[2] == 0x56 and
    buf[3] == 0x01;
}

/// 与えられたバッファがMP4動画データであるかどうかを返します。
pub fn isMp4(buf: []const u8) bool {
  return buf.len > 11 and
    (buf[4] == 'f' and buf[5] == 't' and buf[6] == 'y' and buf[7] == 'p') and
    ((buf[8] == 'a' and buf[9] == 'v' and buf[10] == 'c' and buf[11] == '1') or
      (buf[8] == 'd' and buf[9] == 'a' and buf[10] == 's' and buf[11] == 'h') or
      (buf[8] == 'i' and buf[9] == 's' and buf[10] == 'o' and buf[11] == '2') or
      (buf[8] == 'i' and buf[9] == 's' and buf[10] == 'o' and buf[11] == '3') or
      (buf[8] == 'i' and buf[9] == 's' and buf[10] == 'o' and buf[11] == '4') or
      (buf[8] == 'i' and buf[9] == 's' and buf[10] == 'o' and buf[11] == '5') or
      (buf[8] == 'i' and buf[9] == 's' and buf[10] == 'o' and buf[11] == '6') or
      (buf[8] == 'i' and buf[9] == 's' and buf[10] == 'o' and buf[11] == 'm') or
      (buf[8] == 'm' and buf[9] == 'm' and buf[10] == 'p' and buf[11] == '4') or
      (buf[8] == 'm' and buf[9] == 'p' and buf[10] == '4' and buf[11] == '1') or
      (buf[8] == 'm' and buf[9] == 'p' and buf[10] == '4' and buf[11] == '2') or
      (buf[8] == 'm' and buf[9] == 'p' and buf[10] == '4' and buf[11] == 'v') or
      (buf[8] == 'm' and buf[9] == 'p' and buf[10] == '7' and buf[11] == '1') or
      (buf[8] == 'M' and buf[9] == 'S' and buf[10] == 'N' and buf[11] == 'V') or
      (buf[8] == 'N' and buf[9] == 'D' and buf[10] == 'A' and buf[11] == 'S') or
      (buf[8] == 'N' and buf[9] == 'D' and buf[10] == 'S' and buf[11] == 'C') or
      (buf[8] == 'N' and buf[9] == 'S' and buf[10] == 'D' and buf[11] == 'C') or
      (buf[8] == 'N' and buf[9] == 'D' and buf[10] == 'S' and buf[11] == 'H') or
      (buf[8] == 'N' and buf[9] == 'D' and buf[10] == 'S' and buf[11] == 'M') or
      (buf[8] == 'N' and buf[9] == 'D' and buf[10] == 'S' and buf[11] == 'P') or
      (buf[8] == 'N' and buf[9] == 'D' and buf[10] == 'S' and buf[11] == 'S') or
      (buf[8] == 'N' and buf[9] == 'D' and buf[10] == 'X' and buf[11] == 'C') or
      (buf[8] == 'N' and buf[9] == 'D' and buf[10] == 'X' and buf[11] == 'H') or
      (buf[8] == 'N' and buf[9] == 'D' and buf[10] == 'X' and buf[11] == 'M') or
      (buf[8] == 'N' and buf[9] == 'D' and buf[10] == 'X' and buf[11] == 'P') or
      (buf[8] == 'N' and buf[9] == 'D' and buf[10] == 'X' and buf[11] == 'S') or
      (buf[8] == 'F' and buf[9] == '4' and buf[10] == 'V' and buf[11] == ' ') or
      (buf[8] == 'F' and buf[9] == '4' and buf[10] == 'P' and buf[11] == ' '));
}

test "is m4v" {
    var m4vBuf = [_]u8{0} ** 11;
    m4vBuf[4] = 0x66;
    m4vBuf[5] = 0x74;
    m4vBuf[6] = 0x79;
    m4vBuf[7] = 0x70;
    m4vBuf[8] = 0x4D;
    m4vBuf[9] = 0x34;
    m4vBuf[10] = 0x56;
    try std.testing.expect(isM4v(&m4vBuf));

    var invalid = m4vBuf;
    invalid[8] = 0xFF;
    try std.testing.expect(!isM4v(&invalid));

    const short = [_]u8{ 0x00, 0x00, 0x00, 0x00, 0x66, 0x74, 0x79, 0x70, 0x4D, 0x34 };
    try std.testing.expect(!isM4v(&short));

    const empty = [_]u8{};
    try std.testing.expect(!isM4v(&empty));
}

test "is mkv" {
    var mkv_short = [_]u8{0} ** 16;
    mkv_short[0] = 0x1A;
    mkv_short[1] = 0x45;
    mkv_short[2] = 0xDF;
    mkv_short[3] = 0xA3;
    mkv_short[4] = 0x93;
    mkv_short[5] = 0x42;
    mkv_short[6] = 0x82;
    mkv_short[7] = 0x88;
    mkv_short[8] = 0x6D;
    mkv_short[9] = 0x61;
    mkv_short[10] = 0x74;
    mkv_short[11] = 0x72;
    mkv_short[12] = 0x6F;
    mkv_short[13] = 0x73;
    mkv_short[14] = 0x6B;
    mkv_short[15] = 0x61;
    try std.testing.expect(isMkv(&mkv_short));

    var mkv_longBuf = [_]u8{0} ** 39;
    mkv_longBuf[31] = 0x6D;
    mkv_longBuf[32] = 0x61;
    mkv_longBuf[33] = 0x74;
    mkv_longBuf[34] = 0x72;
    mkv_longBuf[35] = 0x6F;
    mkv_longBuf[36] = 0x73;
    mkv_longBuf[37] = 0x6B;
    mkv_longBuf[38] = 0x61;
    try std.testing.expect(isMkv(&mkv_longBuf));

    var invalid = mkv_short;
    invalid[8] = 0xFF;
    try std.testing.expect(!isMkv(&invalid));

    const short = [_]u8{ 0x1A, 0x45, 0xDF, 0xA3 };
    try std.testing.expect(!isMkv(&short));

    const empty = [_]u8{};
    try std.testing.expect(!isMkv(&empty));
}

test "is webm" {
    const webmBuf = [_]u8{ 0x1A, 0x45, 0xDF, 0xA3, 0x00 };
    try std.testing.expect(isWebm(&webmBuf));

    const invalid = [_]u8{ 0x1A, 0x45, 0xDF, 0xFF };
    try std.testing.expect(!isWebm(&invalid));

    const short = [_]u8{ 0x1A, 0x45, 0xDF };
    try std.testing.expect(!isWebm(&short));

    const empty = [_]u8{};
    try std.testing.expect(!isWebm(&empty));
}

test "is mov" {
    var mov_ftypBuf = [_]u8{0} ** 16;
    mov_ftypBuf[4] = 'f';
    mov_ftypBuf[5] = 't';
    mov_ftypBuf[6] = 'y';
    mov_ftypBuf[7] = 'p';
    mov_ftypBuf[8] = 'q';
    mov_ftypBuf[9] = 't';
    mov_ftypBuf[10] = ' ';
    mov_ftypBuf[11] = ' ';
    try std.testing.expect(isMov(&mov_ftypBuf));

    var mov_moovBuf = [_]u8{0} ** 16;
    mov_moovBuf[4] = 0x6D;
    mov_moovBuf[5] = 0x6F;
    mov_moovBuf[6] = 0x6F;
    mov_moovBuf[7] = 0x76;
    try std.testing.expect(isMov(&mov_moovBuf));

    var invalid = mov_ftypBuf;
    invalid[8] = 'x';
    try std.testing.expect(!isMov(&invalid));

    const short = [_]u8{ 0x00, 0x00, 0x00, 0x00, 'f', 't', 'y', 'p', 'q', 't', ' ', ' ' };
    try std.testing.expect(!isMov(&short));

    const empty = [_]u8{};
    try std.testing.expect(!isMov(&empty));
}

test "is avi" {
    var aviBuf = [_]u8{0} ** 11;
    aviBuf[0] = 0x52;
    aviBuf[1] = 0x49;
    aviBuf[2] = 0x46;
    aviBuf[3] = 0x46;
    aviBuf[8] = 0x41;
    aviBuf[9] = 0x56;
    aviBuf[10] = 0x49;
    try std.testing.expect(isAvi(&aviBuf));

    var invalid = aviBuf;
    invalid[8] = 0xFF;
    try std.testing.expect(!isAvi(&invalid));

    const short = [_]u8{ 0x52, 0x49, 0x46, 0x46 };
    try std.testing.expect(!isAvi(&short));

    const empty = [_]u8{};
    try std.testing.expect(!isAvi(&empty));
}

test "is wmv" {
    const wmvBuf = [_]u8{ 0x30, 0x26, 0xB2, 0x75, 0x8E, 0x66, 0xCF, 0x11, 0xA6, 0xD9, 0x00 };
    try std.testing.expect(isWmv(&wmvBuf));

    const invalid = [_]u8{ 0x30, 0x26, 0xB2, 0x75, 0x8E, 0x66, 0xCF, 0x11, 0xFF, 0xD9 };
    try std.testing.expect(!isWmv(&invalid));

    const short = [_]u8{ 0x30, 0x26, 0xB2, 0x75, 0x8E, 0x66, 0xCF, 0x11, 0xA6 };
    try std.testing.expect(!isWmv(&short));

    const empty = [_]u8{};
    try std.testing.expect(!isWmv(&empty));
}

test "is mpeg" {
    const mpegBuf = [_]u8{ 0x00, 0x00, 0x01, 0xB3, 0x00 };
    try std.testing.expect(isMpeg(&mpegBuf));

    const invalid = [_]u8{ 0x00, 0x00, 0x01, 0xFF };
    try std.testing.expect(!isMpeg(&invalid));

    const short = [_]u8{ 0x00, 0x00, 0x01 };
    try std.testing.expect(!isMpeg(&short));

    const empty = [_]u8{};
    try std.testing.expect(!isMpeg(&empty));
}

test "is flv" {
    const flvBuf = [_]u8{ 0x46, 0x4C, 0x56, 0x01, 0x00 };
    try std.testing.expect(isFlv(&flvBuf));

    const invalid = [_]u8{ 0x46, 0x4C, 0x56, 0xFF };
    try std.testing.expect(!isFlv(&invalid));

    const short = [_]u8{ 0x46, 0x4C, 0x56 };
    try std.testing.expect(!isFlv(&short));

    const empty = [_]u8{};
    try std.testing.expect(!isFlv(&empty));
}

test "is mp4" {
    var mp4Buf = [_]u8{0} ** 12;
    mp4Buf[4] = 'f';
    mp4Buf[5] = 't';
    mp4Buf[6] = 'y';
    mp4Buf[7] = 'p';
    mp4Buf[8] = 'i';
    mp4Buf[9] = 's';
    mp4Buf[10] = 'o';
    mp4Buf[11] = 'm';
    try std.testing.expect(isMp4(&mp4Buf));

    var mp4_mp42Buf = [_]u8{0} ** 12;
    mp4_mp42Buf[4] = 'f';
    mp4_mp42Buf[5] = 't';
    mp4_mp42Buf[6] = 'y';
    mp4_mp42Buf[7] = 'p';
    mp4_mp42Buf[8] = 'm';
    mp4_mp42Buf[9] = 'p';
    mp4_mp42Buf[10] = '4';
    mp4_mp42Buf[11] = '2';
    try std.testing.expect(isMp4(&mp4_mp42Buf));

    var invalid = mp4Buf;
    invalid[8] = 'x';
    try std.testing.expect(!isMp4(&invalid));

    const short = [_]u8{ 0x00, 0x00, 0x00, 0x00, 'f', 't', 'y', 'p', 'i', 's', 'o' };
    try std.testing.expect(!isMp4(&short));

    const empty = [_]u8{};
    try std.testing.expect(!isMp4(&empty));
}